//----------------------------------------------------------------------------
// LptUnit.h
//----------------------------------------------------------------------------


#ifndef LptUnitHPP
#define LptUnitHPP

//----------------------------------------------------------------------------
#include <Controls.hpp>
#include <Graphics.hpp>
#include <Classes.hpp>
#include <SysUtils.hpp>
#include <Messages.hpp>
#include <Windows.hpp>
#include <ExtCtrls.hpp>
#include <System.hpp>

#define MAX_SHARED_BUFFER_LENGTH  127


#define  MaxMappedAreas            16
#define  MaxPins                   17

#define LPTACCESS_ERR_NONE             0
#define LPTACCESS_ERR_BAD_PORT         1
#define LPTACCESS_ERR_ACQUIRE_REFUSED  2

#define LPT_NOT_PRESENT          0
#define LPT_SPP_MODE             1
#define LPT_PS2_MODE             2
#define LPT_EPP_MODE             3
#define LPT_FAST_CENTRONIX       4
#define LPT_ECP_MODE             5

#define COMPATIBILTY_TRANSFER_MODE 0
#define NIBBLE_TRANSFER_MODE       1
#define BYTE_TRANSFER_MODE         2
#define EPP_TRANSFER_MODE          3
#define ECP_TRANSFER_MODE          4


#define EOL "\n"


#define F_D_P  0x00008000

#define MAP_MEMORY        CTL_CODE(F_D_P, 0x802, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define UNMAP_MEMORY      CTL_CODE(F_D_P, 0x803, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define LOCK_MEMORY       CTL_CODE(F_D_P, 0x804, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define UNLOCK_MEMORY     CTL_CODE(F_D_P, 0x805, METHOD_BUFFERED, FILE_ANY_ACCESS) 

#define GET_IRQ_COUNTER   CTL_CODE(F_D_P, 0x806, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define MASK_INT_VEC      CTL_CODE(F_D_P, 0x807, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define UNMASK_INT_VEC    CTL_CODE(F_D_P, 0x808, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define PULSE_EVENT       CTL_CODE(F_D_P, 0x809, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define ACQUIRE_LPT       CTL_CODE(F_D_P, 0x824, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define RELEASE_LPT       CTL_CODE(F_D_P, 0x825, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define CLEAR_LIOPM       CTL_CODE(F_D_P, 0x826, METHOD_BUFFERED, FILE_ANY_ACCESS )
#define SET_RANGE         CTL_CODE(F_D_P, 0x827, METHOD_BUFFERED, FILE_ANY_ACCESS )
#define GET_LIOPMA        CTL_CODE(F_D_P, 0x828, METHOD_BUFFERED, FILE_ANY_ACCESS )
#define ACTIVATE_KIOPM    CTL_CODE(F_D_P, 0x829, METHOD_BUFFERED, FILE_ANY_ACCESS )
#define DEACTIVATE_KIOPM  CTL_CODE(F_D_P, 0x82A, METHOD_BUFFERED, FILE_ANY_ACCESS )
#define QUERY_KIOPM       CTL_CODE(F_D_P, 0x82B, METHOD_BUFFERED, FILE_ANY_ACCESS )


//====================================================================
//==============            TVicLpt 1.3               ================
//==            Declaration of the FindLPT ++ class                ==

const	ResType_IO    = 0x00000002;      // Physical IO address resource.
const	ResType_DMA   = 0x00000003;      // DMA channels 0-7 resource.
const	ResType_IRQ   = 0x00000004;      // IRQ 0-15 resource.

struct  TPort95Config
{
    DWORD LengthRecord;
    DWORD TypeRecord;   	// ResType_IO   = $00000002;
    WORD IOD_Count;
    WORD IOD_Type;
    WORD IOD_Alloc_Base;
    WORD IOD_Alloc_End;
    WORD IOD_DesFlags;
    BYTE IOD_Alloc_Alias;
    BYTE IOD_Alloc_Decode;
    WORD IOR_Align;			// Mask for base alignment
    WORD IOR_nPorts;		// Number of ports
    WORD IOR_Min;			// Min port address
    WORD IOR_Max;			// Max port address
    WORD IOR_RangeFlags;	// Flags
    BYTE IOR_Alias;			// Alias offset
    BYTE IOR_Decode;		// Address specified
};

struct TDMA95Config
{
	DWORD LengthRecord;
	DWORD TypeRecord;   // ResType_DMA   = $00000003;
	BYTE DD_Flags;
	BYTE DD_Alloc_Chan;	// Channel number allocated
	BYTE DD_Req_Mask;
	BYTE DD_Reserved;
};

struct TIRQ95Config
{
	DWORD LengthRecord;
	DWORD TypeRecord;       	// ResType_IRQ   = $00000004;
	WORD IRQD_Flags;
	WORD IRQD_Alloc_Num;		// Allocated IRQ number
	WORD IRQD_Req_Mask;
	WORD IRQD_Reserved;
};

//========================================================================
//======================    Windows NT/2000/XP   =========================
//========================================================================

const CmResourceTypePort             =  1;
const CmResourceTypeInterrupt        =  2;
const CmResourceTypeMemory           =  3;
const CmResourceTypeDma              =  4;

typedef long int PHYSICAL_ADDRESS[2];

struct TPortResource_NT
{
    PHYSICAL_ADDRESS Start;
    long int Length;
};

struct TInterruptResource_NT
{
    long int Level;
    long int Vector;
    long int Affinity;
};

struct TDMAResource_NT
{
    long int Channel;
    long int PortAddr;
    long int Reserved1;
};

struct CM_PARTIAL_RESOURCE_DESCRIPTOR
{
    BYTE TypeResource;  	// Specifies type of resource
    BYTE ShareDisposition;  // Specifies how this resourse can be shared
    WORD Flags;  			// Specifies type-dependent information

	union
	{
		TPortResource_NT PortResource;
		TInterruptResource_NT InterruptResource;
		TDMAResource_NT DMAResource;
	};
};

struct CM_PARTIAL_RESOURCE_LIST
{
    WORD Version;           // The version number of this structure.
    WORD Revision;          // The revision of this structure.
    long int Count;        	// The number of partial resource descriptors
    CM_PARTIAL_RESOURCE_DESCRIPTOR PartialDescriptors[8];
};

struct CM_FULL_RESOURCE_DESCRIPTOR
{
    DWORD InterfaceType;      	// Specifies the bus interface type
    DWORD BusNumber;      		// The zero-based number for this bus
    CM_PARTIAL_RESOURCE_LIST PartialResourceList;
};

struct CM_RESOURCE_LIST
{
    long int Count;              // The number of full resource descriptors
    CM_FULL_RESOURCE_DESCRIPTOR List[8];
};


//===========================================================================
//============    TFindLpt Class Declarations      ==========================
//===========================================================================

class TFindLpt

{
private:

	BOOL m_Win95;
	BOOL m_NT5;
	char m_ClassN[256];
    WORD m_Devices;
    BYTE m_IRQCounter;
	BYTE m_IRQNumber;
    BYTE m_DMACounter;
    BYTE m_DMAChannel;
    BYTE m_PortCounter;
    WORD m_PortBase[2];
    WORD m_PortEnd[2];

    BOOL SearchLpt_9X();
    BOOL  SearchLpt_NT();

public:
	TFindLpt();
    BOOL SearchLpt(BYTE Port_Number);
	BYTE GetIRQCounter();
    BYTE GetIRQNumber();
	BYTE GetDMACounter();
    BYTE GetDMAChannel();
	BYTE GetPortCounter();
    WORD GetPortBase(BYTE Index);
	WORD GetPortEnd(BYTE Index);
};

void OpenSubKey(HKEY MainKey,
				const char *SubKey,
                HKEY* hSubKey,
                DWORD* cSubKeys,
                DWORD* NumKeys);

BOOL SearchKeyValue(HKEY hSubKey,
                    DWORD NumKeys,
                    const char *KeyName,
                    unsigned char *OutBuffer);

//==========================end====================


//-- type declarations -------------------------------------------------------
typedef HANDLE __stdcall (*TOpenVxdHandle)(HANDLE hSmth);

typedef void __fastcall (__closure *TOnHwInterrupt)(BYTE DataPort, BYTE StatusPort, Cardinal TimeStampLo, Cardinal TimeStampHi);

struct TSaveLptState;
typedef TSaveLptState *pSaveLptState;

#pragma pack(push, 1)
struct TSaveLptState
{
    bool sSaved;
    BYTE sMaxLptMode;
    BYTE sCurrLptMode;
    bool sLptReadMode;
    bool sEPPEnabled;
    BYTE sLptAcquired;
    BYTE sDataReadPorts[11];
    BYTE sDataWritePorts[11];
} ;
#pragma pack(pop)

struct TSRec;
typedef TSRec *pSRec;

struct TSRec
{
    DWORD LoTimeStamp;
    DWORD HiTimeStamp;
    BYTE DataValue;
    BYTE StatusValue;
    WORD Rezerved;
} ;

struct TSharedBuffer;
typedef TSharedBuffer *pSharedBuffer;

struct TSharedBuffer
{
    DWORD IrqCounter;
    WORD Rezerved;
    WORD L_Buff;
    WORD N_Add;
    WORD N_Sel;
    TSRec Buff[128];
} ;

struct TLockedBuffer;
typedef TLockedBuffer *pLockedBuffer;

struct TLockedBuffer
{
    TSharedBuffer *pBuf;
    DWORD BufferSize;
    DWORD NumLockedPages;
    TSharedBuffer *pInternalLockedAddress;
} ;

struct TIntRec;
typedef TIntRec *pIntRec;

struct TIntRec
{
    DWORD IRQNumber;
    HANDLE hUserEvent;
    TSharedBuffer *pKernelSharedBuffer;
    DWORD IrqShared;
    BYTE ClearIrq;
    BYTE TypeOfRegister;
    BYTE WideOfRegister;
    BYTE ReadOrWrite;
    DWORD RegBaseAddress;
    DWORD RegOffset;
    DWORD ValueToWrite;
} ;

#ifdef VER95
class PACKAGE TVicLPT : public TComponent
#else
class TVicLPT : public TComponent
#endif

{

private:
	HANDLE           FhDrv;
	HANDLE           FschSCManager;
	HANDLE           FhDll;
	DWORD            FMainDebugCode;
	bool             FWin95;
	BYTE            FLptNumber;
	BYTE            FLpts;
	WORD           FLptBasePort;
	BYTE            FMaxLptMode;
	BYTE            FCurrLptMode;
	bool             FEPPEnabled;
	WORD           FLptAddresses[7];
    		//===  Added with FindLpt ===
    BYTE 			FLptIrq[7];
    BYTE 			FLptDMA[7];
    bool			FPresent[7];
    bool			FLptPresent;
    TFindLpt 		        FFindLpt;
    BYTE 			FDMAChannel;
            //=== end ===


	BYTE            FLptReadMode;
	BYTE            FDataWritePorts[11];
	BYTE            FDataReadPorts[11];
	TSaveLptState    FSaveArea[7];
	BYTE            FLptAcquired;
	HANDLE           FhLptAcquired;
	WORD         * FMappedLPTArea;
	WORD           FTransferDirection;
	WORD           FTransferMode;
	bool             FUseIrq;
	TOpenVxdHandle   FOpenVxDHandle;
	bool             FTerminated;
	TSharedBuffer  * FpLockedMemory;
	TSharedBuffer  * FpInternalLockedMemory;
	TOnHwInterrupt   FOnHwInterrupt;
	WORD           FIrqNumber;
	HANDLE           FLocEvent;
	HANDLE           FKernelEvent;
	HANDLE           FEventIRQSet;
	bool             FOpenDrive;
	BYTE            FDriverInstance;
	bool             FMasked;
	DWORD            FThreadId;
	HANDLE           FThreadHandle;
	DWORD            FLastHandled;
	DWORD            FLastCounter;

	bool __fastcall   InstallDriver(void);
	bool __fastcall   StartDriver(void);
	void __fastcall   CloseStopUnloadDriver(void);
	void __fastcall   InstallStartLoadDriver(BYTE DriverInstance);
	bool __fastcall   OpenTVicLPT(BYTE DriverInstance);
	void __fastcall   CloseTVicLPT(void);
	BYTE __fastcall  GetPort(WORD PortAddress);
	void __fastcall   SetPort(WORD PortAddress, BYTE nNewValue);
	WORD __fastcall GetPortW(WORD PortAddress);
	void __fastcall   SetPortW(WORD PortAddress, WORD nNewValue);
	DWORD __fastcall  GetPortL(WORD PortAddress);
	void __fastcall   SetPortL(WORD PortAddress, DWORD nNewValue);
	BYTE __fastcall  BitRead(BYTE PortData, BYTE nBit);
	BYTE __fastcall  ReadEppTimeoutBit(void);
	bool __fastcall   TestForECP(void);
	bool __fastcall   TestForPS2(void);
	bool __fastcall   TestForSPP(void);
	bool __fastcall   TestForEPP(void);
	BYTE __fastcall  GetIrqNumber(void);
	void __fastcall   SetIrqNumber(BYTE nNewValue);
	DWORD __fastcall  GetIrqCounter(void);
	BYTE __fastcall  GetNumLPTs(void);
	BYTE __fastcall  GetCurrentLPT(void);
	void __fastcall   SetCurrentLPT(BYTE nNewValue);
	BYTE __fastcall  GetReadMode(void);
	void __fastcall   SetReadMode(BYTE nNewValue);
	DWORD __fastcall  GetDebugCode(void);
	WORD __fastcall GetBasePortAddress(void);
	BYTE __fastcall  GetDataPort(void);
	void __fastcall   SetDataPort(BYTE nNewValue);
	BYTE __fastcall  GetStatusPort(void);
	void __fastcall   SetStatusPort(BYTE nNewValue);
	BYTE __fastcall  GetControlPort(void);
	void __fastcall   SetControlPort(BYTE nNewValue);
	BYTE __fastcall  GetEPPAddressPort(void);
	void __fastcall   SetEPPAddressPort(BYTE nNewValue);
	BYTE __fastcall  GetEPPDataPort(void);
	void __fastcall   SetEPPDataPort(BYTE nNewValue);
	BYTE __fastcall  GetAcquired(void);
	void __fastcall   SetAcquired(BYTE nNewValue);
	BYTE __fastcall  GetCurrentLptMode(void);
	void __fastcall   SetCurrentLptMode(BYTE nNewValue);
	BYTE __fastcall  GetMaxLptMode(void);
	bool __fastcall   GetEPPEnabled(void);
	void __fastcall   SetEPPEnabled(bool nNewValue);
	BYTE __fastcall  GetTransferMode(void);
	void __fastcall   SetTransferMode(BYTE nNewValue);
	bool __fastcall   GetIsPrinterReady(void);
	bool  __fastcall  GetActive(void);
	void __fastcall   SetActive(bool nNewValue);
	BYTE __fastcall  GetTransferDirection(void);
	void __fastcall   SetTransferDirection(BYTE nNewValue);
	BYTE __fastcall  GetCfgbPort(void);
	void __fastcall   SetCfgbPort(BYTE nNewValue);
	BYTE __fastcall  GetEcrPort(void);
	void __fastcall   SetEcrPort(BYTE nNewValue);
	BYTE __fastcall  GetFifoPort(void);
	void __fastcall   SetFifoPort(BYTE nNewValue);
	BYTE __fastcall  GetCfgaPort(void);
	void __fastcall   SetCfgaPort(BYTE nNewValue);
	WORD __fastcall GetEPPDataPortWord(void);
	void __fastcall   SetEPPDataPortWord(WORD nNewValue);
	DWORD __fastcall  GetEPPDataPortLong(void);
	void __fastcall   SetEPPDataPortLong(DWORD nNewValue);
	bool __fastcall   GetPin(BYTE nPin);
	void __fastcall   SetPin(BYTE nPin, bool nNewValue);
	BYTE __fastcall  GetDataPortBit(BYTE BitNumber);
	void __fastcall   SetDataPortBit(BYTE BitNumber, BYTE nNewValue);
	BYTE __fastcall  GetStatusPortBit(BYTE BitNumber);
	void __fastcall   SetStatusPortBit(BYTE BitNumber, BYTE nNewValue);
	BYTE __fastcall  GetControlPortBit(BYTE BitNumber);
	void __fastcall   SetControlPortBit(BYTE BitNumber, BYTE nNewValue);
	BYTE __fastcall  GetCfgaPortBit(BYTE BitNumber);
	void __fastcall   SetCfgaPortBit(BYTE BitNumber, BYTE nNewValue);
	BYTE __fastcall  GetCfgbPortBit(BYTE BitNumber);
	void __fastcall   SetCfgbPortBit(BYTE BitNumber, BYTE nNewValue);
	BYTE __fastcall  GetEcrPortBit(BYTE BitNumber);
	void __fastcall   SetEcrPortBit(BYTE BitNumber, BYTE nNewValue);
	void __fastcall   SaveLptState(BYTE nLpt);
	bool __fastcall   RestoreLptState(BYTE nLpt);
	void __fastcall   SetLptIoAccess(BYTE LptNumber);
    // == added with findLPT==
    bool __fastcall   GetPresent();
    BYTE __fastcall   GetDMAChannel(); 
    // ==end==

public:

	DWORD FLastDebugCode;

	__fastcall TVicLPT(TComponent* Owner);
	__fastcall ~TVicLPT(void);

	void __fastcall  MaskIrq(void);
	void __fastcall  UnmaskIrq(void);
	BYTE __fastcall AddCustomPort(WORD CustomBaseAddress);
	void __fastcall  PulseStrobe(DWORD DelayCycles);
	bool __fastcall  PulseStrobeAndWaitBusy(DWORD DelayCycles);
	void __fastcall  ReadEppByteArray(DWORD NumOfBytes, void *ByteBuffer);
	void __fastcall  ReadEppWordArray(DWORD NumOfWords, void *WordBuffer);
	void __fastcall  ReadEppLongArray(DWORD NumOfDWords, void *LongBuffer);
	void __fastcall  WriteEppByteArray(DWORD NumOfBytes, void *ByteBuffer);
	void __fastcall  WriteEppWordArray(DWORD NumOfWORDs, void *WordBuffer);
	void __fastcall  WriteEppLongArray(DWORD NumOfDWORDs, void *LongBuffer);

	__property Byte   IrqNumber = {read=GetIrqNumber, write=SetIrqNumber, nodefault};
	__property DWORD  IrqCounter = {read=GetIrqCounter, nodefault};
	__property BYTE  NumLPTs = {read=GetNumLPTs, nodefault};
	__property BYTE  CurrentLPT = {read=GetCurrentLPT, write=SetCurrentLPT, nodefault};
	__property BYTE  ReadMode = {read=GetReadMode, write=SetReadMode, nodefault};
	__property DWORD  DebugCode = {read=GetDebugCode, nodefault};
	__property WORD BasePortAddress = {read=GetBasePortAddress, nodefault};
	__property BYTE  DataPort = {read=GetDataPort, write=SetDataPort, nodefault};
	__property BYTE  StatusPort = {read=GetStatusPort, write=SetStatusPort, nodefault};
	__property BYTE  ControlPort = {read=GetControlPort, write=SetControlPort, nodefault};
	__property BYTE  EPPAddressPort = {read=GetEPPAddressPort, write=SetEPPAddressPort, nodefault};
	__property BYTE  EPPDataPort = {read=GetEPPDataPort, write=SetEPPDataPort, nodefault};
	__property BYTE  Acquired = {read=GetAcquired, write=SetAcquired, nodefault};
	__property BYTE  CurrentLptMode = {read=GetCurrentLptMode, write=SetCurrentLptMode, nodefault};
	__property BYTE  MaxLptMode = {read=GetMaxLptMode, nodefault};
	__property bool   EPPEnabled = {read=GetEPPEnabled, write=SetEPPEnabled, nodefault};
	__property BYTE  TransferMode = {read=GetTransferMode, write=SetTransferMode, nodefault};
	__property bool   PrinterReady = {read=GetIsPrinterReady, nodefault};
	__property bool   Active = {read=GetActive, write=SetActive, nodefault};
	__property BYTE  TransferDirection = {read=GetTransferDirection, write=SetTransferDirection, nodefault};
	__property BYTE  CfgbPort = {read=GetCfgbPort, write=SetCfgbPort, nodefault};
	__property BYTE  EcrPort = {read=GetEcrPort, write=SetEcrPort, nodefault};
	__property BYTE  FifoPort = {read=GetFifoPort, write=SetFifoPort, nodefault};
	__property BYTE  CfgaPort = {read=GetCfgaPort, write=SetCfgaPort, nodefault};
	__property WORD EPPDataPortWord = {read=GetEPPDataPortWord, write=SetEPPDataPortWord, nodefault};
	__property DWORD  EPPDataPortLong = {read=GetEPPDataPortLong, write=SetEPPDataPortLong, nodefault};
	__property bool   Pin[BYTE Index] = {read=GetPin, write=SetPin};
	__property BYTE  DataPortBit[BYTE Index] = {read=GetDataPortBit, write=SetDataPortBit};
	__property BYTE  StatusPortBit[BYTE Index] = {read=GetStatusPortBit, write=SetStatusPortBit};
	__property BYTE  ControlPortBit[BYTE Index] = {read=GetControlPortBit, write=SetControlPortBit};
	__property BYTE  CfgaPortBit[BYTE Index] = {read=GetCfgaPortBit, write=SetCfgaPortBit};
	__property BYTE  CfgbPortBit[BYTE Index] = {read=GetCfgbPortBit, write=SetCfgbPortBit};
	__property BYTE  EcrPortBit[BYTE Index] = {read=GetEcrPortBit, write=SetEcrPortBit};
    		// ==added with FindLPT==
    __property bool Present = {read=GetPresent, nodefault};
    __property  BYTE DMAChannel = {read=FDMAChannel, nodefault};
			// ==end==

    	 friend
	  void __stdcall TheIsr(TVicLPT& HW32);

__published:
	__property TOnHwInterrupt OnLptInterrupt = {read=FOnHwInterrupt, write=FOnHwInterrupt};
};

#endif	// LptUnit
